@@ -175,6 +175,7 @@ convert_keyval(int key)
   case GDK_F10: return UKey_F10;
   case GDK_F11: return UKey_F11;
   case GDK_F12: return UKey_F12;
+  case GDK_Multi_key: return UKey_Multi_key;
   }
 
   if(key >= GDK_KP_0 && key <= GDK_KP_9) {
@@ -614,7 +615,7 @@ select_candidate_cb(GtkWidget *clist,
       gtk_clist_select_row(GTK_CLIST(uic->cwin.clist),
 			   uic->cwin.candidate_index - uic->cwin.layout_begin,
 			   0);
-      return ;
+      return;
     }
 
     uim_set_candidate_index(uic->uc, idx);
