@@ -16,7 +16,7 @@
 (define make-im
   (lambda (name lang encoding init-arg init-context release-context
 		mode key-press key-release reset
-		get-candidate set-candidate-index)
+		get-candidate set-candidate-index prop)
     (let ((im ()))
       (set! im (cons (cons 'name name) im))
       (set! im (cons (cons 'lang lang) im))
@@ -30,17 +30,18 @@
       (set! im (cons (cons 'reset-handler reset) im))
       (set! im (cons (cons 'get-candidate-handler get-candidate) im))
       (set! im (cons (cons 'set-candidate-index-handler set-candidate-index) im))
+      (set! im (cons (cons 'prop-handler prop) im))
       im)))
 
 (define register-im
   (lambda (name lang encoding init-arg init release
 		mode key-press key-release reset
-		get-candidate set-candidate-index)
+		get-candidate set-candidate-index prop)
     (let
 	((im
 	  (make-im name lang encoding init-arg init release
 		   mode key-press key-release reset
-		   get-candidate set-candidate-index)))
+		   get-candidate set-candidate-index prop)))
       (set! current-im im)
       (im-register-im name lang encoding)
       (if (assq name im-list)
@@ -85,6 +86,9 @@
 (define im-set-candidate-index-handler
   (lambda (im)
     (cdr (assq 'set-candidate-index-handler im))))
+(define im-prop-handler
+  (lambda (im)
+    (cdr (assq 'prop-handler im))))
 ;;
 ;; context-management
 ; context := ((id im.data) ...)
@@ -159,6 +163,8 @@
   (lambda (id idx)()))
 (define default-set-candidate-index-handler
   (lambda (id idx)()))
+(define default-prop-handler
+  (lambda (id prop) ()))
 
 (register-im
  'default
@@ -172,7 +178,8 @@
  default-key-release-handler
  default-reset-handler
  default-get-candidate-handler
- default-set-candidate-index-handler)
+ default-set-candidate-index-handler
+ default-prop-handler)
 ;;
 ;; Dispatch
 (define key-press-handler
@@ -207,6 +214,15 @@
 	 (handler (im-mode-handler im)))
       (handler id mode))))
 
+(define prop-handler
+  (lambda (id prop)
+    (let*
+	((c (find-context id))
+	 (im (context-im c))
+	 (handler (im-prop-handler im)))
+      (handler id prop))))
+
+
 (define select-im
   (lambda (name lang)
     (let ((res ()))
@@ -251,7 +267,6 @@
 	 (im (context-im c))
 	 (handler (im-get-candidate-handler im)))
       (im-return-str (handler id idx)))))
-
 (define set-candidate-index
   (lambda (id idx)
     (let*
