@@ -65,6 +65,14 @@
     (generic-backspace-key key key-state)))
 
 ;; access
+(define skk-context-editor
+  (lambda (c)
+    (car (nthcdr 12 c))))
+
+(define skk-context-set-editor!
+  (lambda (c cnt)
+    (set-car! (nthcdr 12 c) cnt)))
+
 (define skk-context-parent-context
   (lambda (c)
     (car (nthcdr 11 c))))
@@ -176,17 +184,17 @@
 (define skk-flush
   (lambda (sc)
     (rk-flush (skk-context-rk-context sc))
+    (if skk-use-recursive-learning?
+	(skk-editor-flush (skk-context-editor sc)))
     (skk-context-set-state! sc 'skk-state-direct)
     (skk-context-set-head! sc '())
     (skk-context-set-okuri-head! sc "")
     (skk-context-set-okuri! sc '())
     (skk-context-set-candidate-op-count! sc 0)
-    (skk-context-set-candidate-window! sc #f)
-    (skk-context-set-child-context! sc ())
-    (skk-context-set-parent-context! sc ())))
+    (skk-context-set-candidate-window! sc #f)))
 
 (define skk-context-new
-  (lambda ()
+  (lambda (id)
     (if (null? skk-dic-init)
 	(begin
 	  (set! skk-dic-init #t)
@@ -194,15 +202,20 @@
 	   (require "skk-editor.scm"))
 	  (skk-lib-dic-open skk-dic-file-name)
 	  (skk-lib-read-personal-dictionary skk-personal-dic-filename)))
-    (let ((c 
+    (let ((sc 
 	   (copy-list 
-	    '(skk-state-latin #t "" "" "" () () () () () () () ()))))
-      (skk-context-set-head! c ())
-      (skk-context-set-rk-context! c
+	    '(skk-state-latin #t "" "" "" () () () () () () () () ()))))
+      (skk-context-set-head! sc ())
+      (skk-context-set-rk-context! sc
 				   (rk-context-new ja-rk-rule #t #f))
-      (skk-flush c)
-      (skk-context-set-state! c 'skk-state-latin)
-      c)))
+      (skk-context-set-child-context! sc ())
+      (skk-context-set-parent-context! sc ())
+      (if skk-use-recursive-learning?
+	  (skk-context-set-editor! sc
+				   (skk-editor-new sc id)))
+      (skk-flush sc)
+      (skk-context-set-state! sc 'skk-state-latin)
+      sc)))
 
 (define skk-make-string
   (lambda (sl kana)
@@ -245,23 +258,31 @@
 
 (define skk-commit-raw
   (lambda (sc id key key-state)
-    (if (not (skk-context-parent-context sc))
+    (if (skk-context-parent-context sc)
+	(skk-editor-commit-raw
+	 (skk-context-editor
+	  (skk-context-parent-context sc)) key key-state)
 	(im-commit-raw id))))
 
+;; commit string
 (define skk-commit
   (lambda (sc id str)
-    (im-commit id str)))
+    (if (skk-context-parent-context sc)
+	(skk-editor-commit
+	 (skk-context-editor (skk-context-parent-context sc))
+	 str)
+	(im-commit id str))))
 
 (define skk-begin-conversion
   (lambda (sc id)
     (let ((res))
-       ;; get residual 'n'
-       (if (= (skk-context-state sc) 'skk-state-kanji)
+      ;; get residual 'n'
+      (if (= (skk-context-state sc) 'skk-state-kanji)
  	  (let ((tmp (rk-push-key-last! (skk-context-rk-context sc))))
  	    (if tmp
  		(skk-context-set-head! sc
  				       (cons tmp (skk-context-head sc))))))
-       ;;
+      ;;
       (set! res
 	    (skk-lib-get-entry
 	     (skk-make-string (skk-context-head sc) #t)
@@ -289,7 +310,8 @@
 	  (im-pushback-preedit id preedit-reverse ""))
       (if (or
 	   (= stat 'skk-state-kanji)
-	   (= stat 'skk-state-okuri))
+	   (= stat 'skk-state-okuri)
+	   (skk-context-child-context sc))
 	  (let ((h (skk-make-string 
 		    (skk-context-head sc)
 		    (skk-context-kana-mode sc))))
@@ -297,7 +319,9 @@
 		(im-pushback-preedit
 		 id preedit-reverse
 		 h))))
-      (if (= stat 'skk-state-converting)
+      (if (and
+	   (= stat 'skk-state-converting)
+	   (not (skk-context-child-context sc)))
 	  (begin
 	    (im-pushback-preedit
 	     id preedit-reverse
@@ -312,9 +336,13 @@
                            (rk-pending rkc))
       ;; child context's preedit
       (if csc
-	  (begin
+	  (let ((editor (skk-context-editor sc)))
 	    (im-pushback-preedit id preedit-reverse "[")
+	    (im-pushback-preedit id preedit-reverse
+				 (skk-editor-get-left-string editor))
 	    (skk-do-update-preedit id csc)
+	    (im-pushback-preedit id preedit-reverse
+				 (skk-editor-get-right-string editor))
 	    (im-pushback-preedit id preedit-reverse "]")
 	    )))))
 
@@ -338,29 +366,37 @@
     (skk-do-update-preedit id (skk-find-root-context sc))
     (im-update-preedit id)))
 
+;; called from skk-editor
+(define skk-commit-editor-context
+  (lambda (sc id str)
+    (skk-flush sc)
+    (skk-context-set-child-context! sc nil)
+    (skk-commit sc id str)))
+
 (define skk-proc-state-direct-no-preedit
   (lambda (key key-state id sc rkc)
-    (if (skk-wide-latin-key key key-state)
-	(begin
-	  (skk-context-set-state! sc 'skk-state-wide-latin)
-	  (rk-flush rkc)
-	  (skk-update-mode id sc)
-	  #f)
-	#t)
-    (if (skk-latin-key key key-state)
-	(begin
-	  (skk-context-set-state! sc 'skk-state-latin)
-	  (rk-flush rkc)
-	  (skk-update-mode id sc)
-	  #f)
-	#t)
-    (if (skk-kana-toggle key key-state)
-	(begin 
-	  (skk-context-kana-toggle sc)
-	  (skk-update-mode id sc)
-	  #f)
-	#t)
-    #t))
+    (and
+     (if (skk-wide-latin-key key key-state)
+	 (begin
+	   (skk-context-set-state! sc 'skk-state-wide-latin)
+	   (rk-flush rkc)
+	   (skk-update-mode id sc)
+	   #f)
+	 #t)
+     (if (skk-latin-key key key-state)
+	 (begin
+	   (skk-context-set-state! sc 'skk-state-latin)
+	   (rk-flush rkc)
+	   (skk-update-mode id sc)
+	   #f)
+	 #t)
+     (if (skk-kana-toggle key key-state)
+	 (begin 
+	   (skk-context-kana-toggle sc)
+	   (skk-update-mode id sc)
+	   #f)
+	 #t)
+     #t)))
 
 (define skk-proc-state-direct
   (lambda (c key key-state)
@@ -442,6 +478,12 @@
 	   (stat (skk-context-state sc))
 	   (res))
       (and
+       (if (skk-kana-toggle key key-state)
+	   (begin 
+	     (skk-context-kana-toggle sc)
+	     (skk-update-mode id sc)
+	     #f)
+	   #t)
        (if (skk-begin-conv-key key key-state)
 	   (begin
 	     (skk-begin-conversion sc id)
@@ -478,13 +520,17 @@
 	   (begin
 	     (skk-context-set-state! sc 'skk-state-okuri)
 	     (skk-update-mode id sc)
-	     (set! key (to-lower-char key))	     
+	     (set! key (to-lower-char key))
 	     (skk-context-set-okuri-head! sc
 					  (charcode->string key))
+	     (let ((tmp (rk-push-key-last! (skk-context-rk-context sc))))
+	       (if tmp
+		   (skk-context-set-head! sc
+					  (cons tmp (skk-context-head sc)))))
 	     #t)
 	   #t)
        (begin
-	 (set! key (to-lower-char key))	     
+	 (set! key (to-lower-char key))  
 	 (set! stat (skk-context-state sc))
 	 (set! res
 	       (rk-push-key!
@@ -499,13 +545,13 @@
 	 (if (and res (= stat 'skk-state-okuri))
 	     (begin
 	       (skk-context-set-okuri! sc
-				      (cons res ()))
+				       (cons res ()))
 	       (skk-begin-conversion sc id)))))
       nil)))
 
 (define skk-setup-child-context
-  (lambda (sc)
-    (let ((csc (skk-context-new)))
+  (lambda (sc id)
+    (let ((csc (skk-context-new id)))
       (skk-context-set-child-context! sc csc)
       (skk-context-set-parent-context! csc sc)
       (skk-context-set-state! csc 'skk-state-direct))))
@@ -553,8 +599,10 @@
     (if (not (skk-get-current-candidate sc))
 	(begin
 	  (skk-context-set-nth! sc 0)
+	  (if (skk-context-candidate-window sc)
+	      (im-end-candidate id))
 	  (if skk-use-recursive-learning?
-	      (skk-setup-child-context sc))))
+	      (skk-setup-child-context sc id))))
     (if (not (skk-context-child-context sc))
 	(begin
 	  ;; Windowɽ򳫻Ϥ뤫
@@ -655,7 +703,6 @@
 	   (begin
 	     (if (not (rk-backspace rkc))
 		 (begin
-		   (print (cdr (skk-context-okuri sc)))
 		   (if (cdr (skk-context-okuri sc))
 		       (skk-context-set-okuri! sc
 			(cdr (skk-context-okuri sc)))
@@ -692,7 +739,7 @@
 
 (define skk-proc-state-wide-latin
   (lambda (c key key-state)
-    (let* ((w (jp-wide (charcode->string key)))
+    (let* ((w (ja-wide (charcode->string key)))
 	   (id (context-id c))
 	   (sc (skk-current-context c)))
       (if (skk-on-key key key-state)
@@ -733,7 +780,7 @@
   (lambda (id arg)
     (let* ((c (find-context id)))
       (set-context-data! c
-			 (skk-context-new))
+			 (skk-context-new id))
       (im-clear-mode-list id)
       (im-pushback-mode-list id "RAW")
       (im-pushback-mode-list id "Ҥ餬")
