@@ -46,6 +46,7 @@ struct skk_line {
 
 static struct dic_info {
   void *addr;
+  int first;
   int border;
   int size;
   struct skk_line head;
@@ -75,6 +76,19 @@ is_okuri(char *str)
   return 0;
 }
 
+
+static int
+find_first(struct dic_info *di)
+{
+  char *s = di->addr;
+  int off = 0;
+  while ( s[off] < di->size + s && s[off] == ';' ) {
+    int l = calc_line_len(&s[off]);
+    off += l + 1;
+  }
+  return off;
+}
+
 static int
 find_border(struct dic_info *di)
 {
@@ -83,7 +97,7 @@ find_border(struct dic_info *di)
   while (1) {
     int l = calc_line_len(&s[off]);
     if (s[off] == ';') {
-      off += l +1;
+      off += l + 1;
       continue;
     }
     if (!is_okuri(&s[off])) {
@@ -116,6 +130,7 @@ open_dic(const char *fn)
   di = (struct dic_info *)malloc(sizeof(struct dic_info));
   di->addr = addr;
   di->size = st.st_size;
+  di->first = find_first(di);
   di->border = find_border(di);
   di->head.next = NULL;
   return di;
@@ -125,7 +140,7 @@ static char *
 find_line(struct dic_info *di, int off)
 {
   char *ptr = di->addr;
-  while (off > 0 && ptr[off] != '\n') {
+  while (off > 0 && (ptr[off] != '\n' || ptr[off+1] == ';')) {
     off --;
   }
   if (off) {
@@ -163,7 +178,7 @@ do_search(struct dic_info *di, char *s, int min,
   }
   r = extract_entry(di, idx, buf, 256);
   if (r) {
-    c = strcmp(s,r);
+    c = strcmp(s, r);
   } else {
     return -1;
   }
@@ -233,6 +248,9 @@ static void
 free_skk_line(struct skk_line *sl)
 {
   int i, j;
+  if (!sl) {
+    return ;
+  }
   for (i = 0; i < sl->nr_cand_array; i++) {
     struct skk_cand_array *ca = &sl->cands[i];
     for (j = 0; j < ca->nr_cands; j++) {
@@ -288,7 +306,11 @@ static void
 merge_candidate_array(struct skk_line *sl, struct skk_cand_array *dst_ca)
 {
   int i, j;
-  struct skk_cand_array *src_ca = &sl->cands[0];
+  struct skk_cand_array *src_ca;
+  if (!sl) {
+    return ;
+  }
+  src_ca = &sl->cands[0];
   if (src_ca == dst_ca) {
     return ;
   }
@@ -383,7 +405,7 @@ skk_search_line_from_file(struct dic_info *di, char *s, char okuri_head)
   }
   sprintf(idx, "%s%c",s, okuri_head);
   if (okuri_head) {
-    n = do_search(di, idx, 0, di->border - 1, -1);
+    n = do_search(di, idx, di->first, di->border - 1, -1);
   } else {
     n = do_search(di, idx, di->border, di->size - 1, 1);
   }
@@ -664,6 +686,34 @@ skk_lib_read_personal_dictionary(LISP fn_)
   return sym_t;
 }
 
+static LISP
+skk_split_string(LISP str_)
+{
+  char *str = get_c_string(str_);
+  unsigned char *cur = str;
+  LISP res = NIL;
+  while (*cur) {
+    LISP tmp;
+    char buf[3];
+    int len;
+    buf[2] = 0;
+    if (*cur > 127) {
+      /* 2 bytes */
+      buf[0] = cur[0];
+      buf[1] = cur[1];
+      len = 2;
+      cur ++;
+    } else {
+      buf[0] = cur[0];
+      buf[1] = 0;
+      len = 1;
+    }
+    res = cons (strcons(len, buf), res);
+    cur ++;
+  }
+  return res;
+}
+
 void uim_init_skk_dic()
 {
   init_subr_1("skk-lib-dic-open", skk_dic_open);
@@ -673,4 +723,5 @@ void uim_init_skk_dic()
   init_subr_4("skk-lib-get-nth-candidate", skk_get_nth_candidate);
   init_subr_3("skk-lib-get-nr-candidates", skk_get_nr_candidates);
   init_subr_4("skk-lib-commit-candidate", skk_commit_candidate);
+  init_subr_1("skk-lib-split-string", skk_split_string);
 }
