@@ -8,6 +8,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <pwd.h>
 #include <errno.h>
 #include <uim/context.h>
 
@@ -73,6 +74,13 @@ uim_helper_get_pathname(void)
   char *login;
  
   login = getenv("LOGNAME");
+  
+  if(!login) {
+    struct passwd *pw;
+    pw = getpwuid(getuid());
+    login = strdup(pw->pw_name);
+    free(pw);
+  }
 
   path = (char*)malloc(strlen(login)+ 20);
   sprintf(path, "/tmp/uimhelper-%s",login);
