@@ -41,6 +41,10 @@ nthcdr(LISP nth_, LISP lst)
   int nth = get_c_long(nth_);
   int i;
   for (i = 0; i < nth; i++) {
+    if (!lst) {
+      /* something bad happened */
+      return NIL;
+    }
     lst = CDR(lst);
   }
   return lst;
